@@ -191,12 +191,20 @@ class SearchTestCase(ModuleTestCase('../redisearch.so')):
                                                 'f', 'hello world'))
 
             for i in range(100):
+                # the doc hash should exist now
+                self.assertTrue(r.exists('doc%d' % i))
+                # Delete the actual docs only half of the time
                 self.assertEqual(1, r.execute_command(
-                    'ft.del', 'idx', 'doc%d' % i))
+                    'ft.del', 'idx', 'doc%d' % i, 'DD' if i % 2 == 0 else ''))
                 # second delete should return 0
                 self.assertEqual(0, r.execute_command(
                     'ft.del', 'idx', 'doc%d' % i))
 
+                # After del with DD the doc hash should not exist
+                if i % 2 == 0:
+                    self.assertFalse(r.exists('doc%d' % i))
+                else:
+                    self.assertTrue(r.exists('doc%d' % i))
                 res = r.execute_command(
                     'ft.search', 'idx', 'hello', 'nocontent', 'limit', 0, 100)
                 self.assertNotIn('doc%d' % i, res)
@@ -265,19 +273,35 @@ class SearchTestCase(ModuleTestCase('../redisearch.so')):
         with self.redis() as r:
             r.flushdb()
             self.assertOk(r.execute_command(
-                'ft.create', 'idx', 'schema', 'f', 'text', 'n', 'numeric', 't', 'tag'))
+                'ft.create', 'idx', 'schema', 'f', 'text', 'n', 'numeric', 't', 'tag', 'g', 'geo'))
 
-            for i in range(200):
+            for i in range(100):
                 self.assertOk(r.execute_command('ft.add', 'idx', 'doc%d' % i, 1.0, 'fields',
-                                                'f', 'hello world', 'n', 666, 't', 'foo bar'))
-
+                                                'f', 'hello world', 'n', 666, 't', 'foo bar',
+                                                'g', '19.04,47.497'))
             keys = r.keys('*')
-            self.assertEqual(205, len(keys))
+            self.assertEqual(106, len(keys))
 
             self.assertOk(r.execute_command('ft.drop', 'idx'))
             keys = r.keys('*')
             self.assertEqual(0, len(keys))
 
+            # Now do the same with KEEPDOCS
+            self.assertOk(r.execute_command(
+                'ft.create', 'idx', 'schema', 'f', 'text', 'n', 'numeric', 't', 'tag', 'g', 'geo'))
+
+            for i in range(100):
+                self.assertOk(r.execute_command('ft.add', 'idx', 'doc%d' % i, 1.0, 'fields',
+                                                'f', 'hello world', 'n', 666, 't', 'foo bar',
+                                                'g', '19.04,47.497'))
+            keys = r.keys('*')
+            self.assertEqual(106, len(keys))
+
+            self.assertOk(r.execute_command('ft.drop', 'idx', 'KEEPDOCS'))
+            keys = r.keys('*')
+            self.assertListEqual(['doc0', 'doc1', 'doc10', 'doc11', 'doc12', 'doc13', 'doc14', 'doc15', 'doc16', 'doc17', 'doc18', 'doc19', 'doc2', 'doc20', 'doc21', 'doc22', 'doc23', 'doc24', 'doc25', 'doc26', 'doc27', 'doc28', 'doc29', 'doc3', 'doc30', 'doc31', 'doc32', 'doc33', 'doc34', 'doc35', 'doc36', 'doc37', 'doc38', 'doc39', 'doc4', 'doc40', 'doc41', 'doc42', 'doc43', 'doc44', 'doc45', 'doc46', 'doc47', 'doc48', 'doc49', 'doc5', 'doc50', 'doc51', 'doc52', 'doc53',
+                                  'doc54', 'doc55', 'doc56', 'doc57', 'doc58', 'doc59', 'doc6', 'doc60', 'doc61', 'doc62', 'doc63', 'doc64', 'doc65', 'doc66', 'doc67', 'doc68', 'doc69', 'doc7', 'doc70', 'doc71', 'doc72', 'doc73', 'doc74', 'doc75', 'doc76', 'doc77', 'doc78', 'doc79', 'doc8', 'doc80', 'doc81', 'doc82', 'doc83', 'doc84', 'doc85', 'doc86', 'doc87', 'doc88', 'doc89', 'doc9', 'doc90', 'doc91', 'doc92', 'doc93', 'doc94', 'doc95', 'doc96', 'doc97', 'doc98', 'doc99'], sorted(keys))
+
     def testCustomStopwords(self):
         with self.redis() as r:
             r.flushdb()
@@ -331,18 +355,18 @@ class SearchTestCase(ModuleTestCase('../redisearch.so')):
         self.assertEqual(0, r1[0])
         self.assertEqual(1, r2[0])
 
-
     def testNoStopwords(self):
         # This test taken from Java's test suite
         self.cmd('ft.create', 'idx', 'schema', 'title', 'text')
         for i in range(100):
             self.cmd('ft.add', 'idx', 'doc{}'.format(i), 1.0, 'fields',
-                'title', 'hello world' if i % 2 == 0 else 'hello worlds')
+                     'title', 'hello world' if i % 2 == 0 else 'hello worlds')
 
         res = self.cmd('ft.search', 'idx', 'hello a world', 'NOCONTENT')
         self.assertEqual(100, res[0])
 
-        res = self.cmd('ft.search', 'idx', 'hello a world', 'VERBATIM', 'NOCONTENT')
+        res = self.cmd('ft.search', 'idx', 'hello a world',
+                       'VERBATIM', 'NOCONTENT')
         self.assertEqual(50, res[0])
 
         res = self.cmd('ft.search', 'idx', 'hello a world', 'NOSTOPWORDS')
@@ -449,7 +473,7 @@ class SearchTestCase(ModuleTestCase('../redisearch.so')):
             res = r.execute_command(
                 'ft.search', 'idx', 'wat', 'nocontent', 'withscores')
             self.assertLess(float(res[2]), 1)
-            #self.assertListEqual([1L, 'doc1'], res)
+            # self.assertListEqual([1L, 'doc1'], res)
             self.assertOk(r.execute_command('ft.add', 'idx',
                                             'doc1', '1.0', 'replace', 'partial', 'fields'))
             res = r.execute_command(
@@ -1708,14 +1732,16 @@ class SearchTestCase(ModuleTestCase('../redisearch.so')):
 
     def testLanguageField(self):
         self.cmd('FT.CREATE', 'idx', 'SCHEMA', 'language', 'TEXT')
-        self.cmd('FT.ADD', 'idx', 'doc1', 1.0, 'FIELDS', 'language', 'gibberish')
+        self.cmd('FT.ADD', 'idx', 'doc1', 1.0,
+                 'FIELDS', 'language', 'gibberish')
         res = self.cmd('FT.SEARCH', 'idx', 'gibberish')
         self.assertEqual([1L, 'doc1', ['language', 'gibberish']], res)
         # The only way I can verify that LANGUAGE is parsed twice is ensuring we
         # provide a wrong language. This is much easier to test than trying to
         # figure out how a given word is stemmed
         with self.assertResponseError():
-            self.cmd('FT.ADD', 'idx', 'doc1', 1.0, 'LANGUAGE', 'blah', 'FIELDS', 'language', 'gibber')
+            self.cmd('FT.ADD', 'idx', 'doc1', 1.0, 'LANGUAGE',
+                     'blah', 'FIELDS', 'language', 'gibber')
 
 
 def grouper(iterable, n, fillvalue=None):
